/// @addtogroup vectalg
/// @{
/////////////////////////////////////////////////////////////////////////////
/// @file IVectorMD.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file constitutes a part of the CAPD library, 
// distributed under the terms of the GNU General Public License.
// Consult  http://capd.wsb-nlu.edu.pl/ for details. 

#include <iostream>
#include "interval/doubleInterval.h"
#include "vectalg/Vector.hpp"

using namespace capd::vectalg;

template class Vector<interval,0>;

template Vector<interval,0> abs<interval> (const Vector<interval,0> &v);
template Vector<interval,0> operator- <interval> (const Vector<interval,0> &v);
template Vector<interval,0> operator+ <interval>(const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template Vector<interval,0> operator- <interval>(const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template interval operator* <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);

template Vector<interval,0> operator* <interval,interval>(const Vector<interval,0> &v,const interval &s);
template Vector<interval,0> operator* <interval,interval>(const interval &s,const Vector<interval,0> &v);
template Vector<interval,0> operator/ <interval,interval>(const Vector<interval,0> &v, const interval &s);

template Vector<interval,0> operator* <interval,double>(const Vector<interval,0> &v,const double &s);
template Vector<interval,0> operator* <interval,double>(const double &s,const Vector<interval,0> &v);
template Vector<interval,0> operator/ <interval,double>(const Vector<interval,0> &v, const double &s);

template Vector<interval,0> operator* <interval,long>(const Vector<interval,0> &v,const long &s);
template Vector<interval,0> operator* <interval,long>(const long &s,const Vector<interval,0> &v);
template Vector<interval,0> operator/ <interval,long>(const Vector<interval,0> &v, const long &s);

template Vector<interval,0> operator* <interval,int>(const Vector<interval,0> &v,const int &s);
template Vector<interval,0> operator* <interval,int>(const int &s,const Vector<interval,0> &v);
template Vector<interval,0> operator/ <interval,int>(const Vector<interval,0> &v, const int &s);

template Vector<interval,0> operator+ <interval> (const Vector<interval,0> &v, const interval &s);
template Vector<interval,0> operator- <interval>(const Vector<interval,0> &v,const interval &s);
template bool operator < <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template bool operator > <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template bool operator<= <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template bool operator>= <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template std::ostream &operator<< <interval> (std::ostream &out, const Vector<interval,0> &v);
template std::istream &operator>> <interval>(std::istream &inp, Vector<interval,0> &v);


typedef Vector<interval,0> IVector;

template IVector intervalHull<IVector>(const IVector &A, const IVector &B);
template void split<IVector>(IVector& v, IVector& rv);
template IVector midVector<IVector>(const IVector& v);
template IVector leftVector<IVector>(const IVector& v);
template IVector rightVector<IVector>(const IVector& v);
template interval size<IVector>(const IVector& v);
template IVector diam<IVector>(const IVector& v);
template IVector intervalBall<IVector>(const IVector &iv, const interval &r);
template interval solveAffineInclusion<IVector>(const IVector& a,const IVector& p,const IVector& c);
template interval solveAffineInclusion<IVector>(const IVector& a,const IVector& p,const IVector& c,int&);
template bool subset<IVector>(const IVector& v1, const IVector& v2);
template bool subsetInterior<IVector>(const IVector& v1, const IVector& v2);
template IVector intersection<IVector>(const IVector &v1, const IVector &v2);

/// @}
